package net.kronos.mockup.handler;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import net.kronos.mockup.annotation.MockAddress;
import net.kronos.orm.rowset.ExceptionHandler;

public class AddressHandler implements  IMockHandlerCommand{
	private boolean isInitialized = false;
//	private static List<String[]> csvData = new ArrayList<String[]>();
	private static Map<String, List<String[]>> prefectureMap = new HashMap<String, List<String[]>>();
	public AddressHandler(){
		// 初期化は初めだけ
		if(this.isInitialized)return;
		// 初期化
		readAddressCSV();
		this.isInitialized = true;
	}
	
	public static final int COL_JIS = 0;
	public static final int COL_JIGYOSYO_KANA = 1;
	public static final int COL_JIGYOSYO = 2;
	public static final int COL_PREFECTURE = 3;
	public static final int COL_CITY = 4;
	public static final int COL_TOWN = 5;
	public static final int COL_STREET = 6;
	
	/**
	 * 
	// 01101,
	// "(ｶﾌﾞ) ﾆﾎﾝｹｲｻﾞｲｼﾝﾌﾞﾝｼﾔ ｻﾂﾎﾟﾛｼｼﾔ",
	// "株式会社　日本経済新聞社　札幌支社",
	// "北海道",
	// "札幌市中央区",
	// "北一条西",
	// "６丁目１－２アーバンネット札幌ビル２Ｆ",
	// "0608621",
	// "060  ",
	// "札幌",
	// 0,
	// 0,
	// 0
	 */
	protected void readAddressCSV(){
		try {
			InputStream in = this.getClass().getResourceAsStream("/net/kronos/mockup/JIGYOSYO.csv");
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String line;
			while((line = br.readLine()) != null){
				String[] cols = line.replaceAll("\"", "").split(",");

				String pref = cols[COL_PREFECTURE];
				List<String[]> prefList = new ArrayList<String[]>();
				
				// 都道府県のMapを取得
				if(prefectureMap.containsKey(pref)){
					prefList = prefectureMap.get(pref);
				}
				
				prefList.add(cols);
				prefectureMap.put(pref,prefList);
				
			}
			br.close();
			in.close();
			return;
		} catch (FileNotFoundException e) {
			throw ExceptionHandler.wrapNoCheckException(e);
		} catch (IOException e) {
			throw ExceptionHandler.wrapNoCheckException(e);
		}
	}

	/** 現在のカウントに対応するCSV行を保持する */
	private String[] currentRow;
	private int currentCount;
	
	protected String[] getCsvData(int count){
		int prefNum = count % prefectureMap.size();
		if(count == 0 || currentCount != count){
			String[] keys ={};
			keys = prefectureMap.keySet().toArray(keys);
			
			List<String[]> list = prefectureMap.get(keys[prefNum]);
			int index = NumberHandler.getRandomNumber(list.size()-1);
			currentRow = list.get(index);
			currentCount = count;
		}
		return currentRow;
	}
	
	
	@Override
	public Object execute(
			Annotation ann, 
			Field targetField, 
			Object instance,
			int count) {
		MockAddress addAnn = (MockAddress)ann;
		
		AddressData data = new AddressData(getCsvData(count));
		switch(addAnn.pattern()){
		case PostNumber : 
			return data.getJis();
		case Prefecture : 
			return data.getPrefecture();
		case City : 
			return data.getCity();
		case Town : 
			return data.getTown();
		case Street : 
			return data.getStreet();
		case BusinessPlace :
			return data.getJigyosyo();
		default :
			return data.getFullAddress();
		}
	}
	
	public class AddressData{
		private String[] csvData;
		public AddressData(String[] csvData){
			this.csvData = csvData;
		}
		public String getJis(){
			return this.csvData[COL_JIS];
		}
		public String getJigyosyo(){
			return this.csvData[COL_JIGYOSYO];
		}
		public String getJigyosyoKana(){
			return this.csvData[COL_JIGYOSYO_KANA];
		}
		public String getPrefecture(){
			return this.csvData[COL_PREFECTURE];
		}
		public String getCity(){
			return this.csvData[COL_CITY];
		}
		public String getTown(){
			return this.csvData[COL_TOWN];
		}
		public String getStreet(){
			return this.csvData[COL_STREET];
		}
		public String getFullAddress(){
			return new StringBuffer()
			.append(this.getPrefecture())
			.append(this.getCity())
			.append(this.getTown())
			.append(this.getStreet()).toString();
		}

	}
	

}
